@@ -478,6 +478,9 @@ sub __deliver {
 
 	# loop until leg und dst_addr are known, when we call leg->deliver
 	my $leg = $qentry->{leg}[0];
+	if ( $leg && @{ $qentry->{leg}}>1 ) {
+		DEBUG( 50,"picking first of multiple legs: ".join( " ", map { $_->dump } @{ $qentry->{leg}} ));
+	}
 	my $dst_addr = $qentry->{dst_addr}[0];
 
 	if ( ! $dst_addr || ! $leg) {
@@ -526,7 +529,7 @@ sub __deliver {
 
 	# adds via on cloned packet, calls cb if definite success (tcp)
 	# or error
-	DEBUG( 50,"deliver through leg $leg \@$dst_addr" );
+	DEBUG( 50,"deliver through leg ".$leg->dump." \@$dst_addr" );
 	weaken( my $rself = \$self );
 	$cb = [ $cb,$self,$qentry ];
 	weaken( $cb->[1] );
@@ -775,16 +778,42 @@ sub dns_domain2srv {
 	my $dns = Net::DNS::Resolver->new;
 
 	# Try to get SRV records for _sip._udp.domain or _sip._tcp.domain
-	my @resp;
+	my (@resp,%addr2ip);
 	foreach my $proto ( @$protos ) {
 		if ( my $q = $dns->query( '_'.$sip_proto.'._'.$proto.'.'.$domain,'SRV' )) {
 			foreach my $rr ( $q->answer ) {
-				$rr->type eq 'SRV' || next;
-				# XXX fixme, get IPs for name
-				push @resp,[ $rr->priority, $proto,$rr->name,$rr->port ]
+				if ( $rr->type eq 'A' ) {
+					push @{ $addr2ip{$rr->name} }, $rr->address;
+				} elsif ( $rr->type eq 'SRV' ) {
+					push @resp,[ $rr->priority, $proto,$rr->target,$rr->port ]
+				}
 			}
 		}
 	}
+
+	# name to addr based on additional records in DNS answer
+	my @resp_resolved;
+	for my $r (@resp) {
+		if ( my $addr = $addr2ip{ $r->[2] } ) {
+			for (@$addr) {
+				my @cp = @$r;
+				$cp[2] = $_;
+				push @resp_resolved, \@cp;
+			}
+		} else {
+			# either already IP or no additional data for resolving -> later
+			my @cp = @$r;
+			# XXX fixme blocking DNS lookup
+			my $ipn = gethostbyname( $r->[2] ) or do {
+				DEBUG( 1,"cannot resolve $r->[2]" );
+				next;
+			};
+			$cp[2] = inet_ntoa($ipn);
+			push @resp_resolved, \@cp;
+		}
+	}
+	@resp = @resp_resolved;
+
 	# if no SRV records try to resolve address directly
 	unless (@resp) {
 		# try addr directly
@@ -792,7 +821,8 @@ sub dns_domain2srv {
 		if ( my $q = $dns->query( $domain,'A' )) {
 			foreach my $rr ($q->answer ) {
 				$rr->type eq 'A' || next;
-				# XXX fixme, get *all* IPs for name
+				# XXX fixme, check that name in response corresponds to query
+				# (beware of CNAMEs!)
 				push @resp,map {
 					[ -1, $_ , $rr->address,$default_port ]
 				} @$protos;
